==> sarus_data_spec_public-3.2.1.dev0.tar.gz: sarus_data_spec_public-3.2.1.dev0/sarus_data_spec/manager/ops/processor/external/numpy.py <==
from typing import Any, List, Literal, Optional, Tuple, Union

from numpy.typing import ArrayLike, DTypeLike
import numpy as np

from sarus_data_spec.dataspec_validator.parameter_kind import DATASPEC, STATIC
from sarus_data_spec.dataspec_validator.signature import (
    SarusBoundSignature,
    SarusParameter,
    SarusSignature,
)

from .external_op import ExternalOpImplementation

Casting = Literal["no", "equiv", "safe", "same_kind", "unsafe"]
Order = Literal["K", "A", "C", "F"]


# ------ CONSTRUCTORS ------
class np_array(ExternalOpImplementation):
    _transform_id = "numpy.NP_ARRAY"
    _signature = SarusSignature(
        SarusParameter(
            name="object",
            annotation=ArrayLike,
            condition=DATASPEC | STATIC,
        ),
        SarusParameter(
            name="dtype",
            annotation=Optional[DTypeLike],
            default=None,
        ),
        SarusParameter(
            name="copy",
            annotation=bool,
            default=True,
        ),
        SarusParameter(
            name="order",
            annotation=Order,
            default="K",
        ),
        SarusParameter(
            name="subok",
            annotation=bool,
            default=False,
        ),
        SarusParameter(
            name="ndmin",
            annotation=int,
            default=0,
        ),
        SarusParameter(
            name="like",
            annotation=Optional[ArrayLike],
            default=None,
        ),
    )

    async def call(self, signature: SarusBoundSignature) -> Any:
        kwargs = await signature.collect_kwargs()
        if kwargs["like"] is None:
            del kwargs["like"]
        return np.array(**kwargs)


# ------ FUNCTIONS ------
class np_ceil(ExternalOpImplementation):
    _transform_id = "numpy.NP_CEIL"
    _signature = SarusSignature(
        SarusParameter(
            name="x",
            annotation=ArrayLike,
            condition=DATASPEC | STATIC,
        ),
        SarusParameter(
            name="out",
            annotation=Optional[ArrayLike],
            default=None,
            predicate=lambda x: x is None,
        ),
        SarusParameter(
            name="where",
            annotation=Optional[Union[bool, ArrayLike]],
            default=True,
        ),
        SarusParameter(
            name="casting",
            annotation=Casting,
            default="same_kind",
        ),
        SarusParameter(
            name="order",
            annotation=Order,
            default="K",
        ),
        SarusParameter(
            name="dtype",
            annotation=Optional[DTypeLike],
            default=None,
        ),
        SarusParameter(
            name="subok",
            annotation=bool,
            default=False,
        ),
    )

    async def call(self, signature: SarusBoundSignature) -> Any:
        x, kwargs = await signature.collect_kwargs_method()
        return np.ceil(x, **kwargs)


class np_floor(ExternalOpImplementation):
    _transform_id = "numpy.NP_FLOOR"
    _signature = SarusSignature(
        SarusParameter(
            name="x",
            annotation=ArrayLike,
            condition=DATASPEC | STATIC,
        ),
        SarusParameter(
            name="out",
            annotation=Optional[ArrayLike],
            default=None,
            predicate=lambda x: x is None,
        ),
        SarusParameter(
            name="where",
            annotation=Optional[Union[bool, ArrayLike]],
            default=True,
        ),
        SarusParameter(
            name="casting",
            annotation=Casting,
            default="same_kind",
        ),
        SarusParameter(
            name="order",
            annotation=Order,
            default="K",
        ),
        SarusParameter(
            name="dtype",
            annotation=Optional[DTypeLike],
            default=None,
        ),
        SarusParameter(
            name="subok",
            annotation=bool,
            default=False,
        ),
    )

    async def call(self, signature: SarusBoundSignature) -> Any:
        x, kwargs = await signature.collect_kwargs_method()
        return np.floor(x, **kwargs)


class np_mean(ExternalOpImplementation):
    _transform_id = "numpy.NP_MEAN"
    _signature = SarusSignature(
        SarusParameter(
            name="a",
            annotation=ArrayLike,
            condition=DATASPEC | STATIC,
        ),
        SarusParameter(
            name="axis",
            annotation=Optional[Union[int, Tuple[int]]],
            default=None,
        ),
        SarusParameter(
            name="dtype",
            annotation=Optional[DTypeLike],
            default=None,
        ),
        SarusParameter(
            name="out",
            annotation=Optional[ArrayLike],
            default=None,
            predicate=lambda x: x is None,
        ),
        SarusParameter(
            name="keepdims",
            annotation=bool,
            default=False,
        ),
        SarusParameter(
            name="where",
            annotation=Optional[Union[bool, ArrayLike]],
            default=True,
        ),
    )

    async def call(self, signature: SarusBoundSignature) -> Any:
        kwargs = await signature.collect_kwargs()
        return np.mean(**kwargs)


class np_std(ExternalOpImplementation):
    _transform_id = "numpy.NP_STD"
    _signature = SarusSignature(
        SarusParameter(
            name="a",
            annotation=ArrayLike,
            condition=DATASPEC | STATIC,
        ),
        SarusParameter(
            name="axis",
            annotation=Optional[Union[int, Tuple[int]]],
            default=None,
        ),
        SarusParameter(
            name="dtype",
            annotation=Optional[DTypeLike],
            default=None,
        ),
        SarusParameter(
            name="out",
            annotation=Optional[ArrayLike],
            default=None,
            predicate=lambda x: x is None,
        ),
        SarusParameter(
            name="ddof",
            annotation=int,
            default=0,
        ),
        SarusParameter(
            name="keepdims",
            annotation=bool,
            default=False,
        ),
        SarusParameter(
            name="where",
            annotation=Optional[Union[bool, ArrayLike]],
            default=True,
        ),
    )

    async def call(self, signature: SarusBoundSignature) -> Any:
        kwargs = await signature.collect_kwargs()
        return np.std(**kwargs)


class np_rand(ExternalOpImplementation):
    _transform_id = "numpy.NP_RAND"
    _signature = SarusSignature(
        SarusParameter(
            name="size",
            annotation=Optional[Union[int, List[int]]],
            default=None,
        ),
    )

    async def call(self, signature: SarusBoundSignature) -> Any:
        kwargs = await signature.collect_kwargs()
        return np.random.random_sample(**kwargs)


# ------ METHODS ------
class np_astype(ExternalOpImplementation):
    _transform_id = "numpy.NP_ASTYPE"
    _signature = SarusSignature(
        SarusParameter(
            name="this",
            annotation=np.ndarray,
            condition=DATASPEC | STATIC,
        ),
        SarusParameter(
            name="dtype",
            annotation=DTypeLike,
        ),
        SarusParameter(
            name="order",
            annotation=Order,
            default="K",
        ),
        SarusParameter(
            name="casting",
            annotation=Casting,
            default="unsafe",
        ),
        SarusParameter(
            name="subok",
            annotation=bool,
            default=False,
        ),
        SarusParameter(
            name="copy",
            annotation=bool,
            default=True,
        ),
    )

    async def call(self, signature: SarusBoundSignature) -> Any:
        this, kwargs = await signature.collect_kwargs_method()
        return this.astype(**kwargs)

==> sarus_data_spec_public-3.3.0.tar.gz: sarus_data_spec_public-3.3.0/sarus_data_spec/manager/ops/processor/external/numpy.py <==
from typing import Any, List, Literal, Optional, Tuple, Union

from numpy.typing import ArrayLike, DTypeLike
import numpy as np

from sarus_data_spec.dataspec_validator.parameter_kind import DATASPEC, STATIC
from sarus_data_spec.dataspec_validator.signature import (
    SarusBoundSignature,
    SarusParameter,
    SarusParameterArray,
    SarusSignature,
)

from .external_op import ExternalOpImplementation

Casting = Literal["no", "equiv", "safe", "same_kind", "unsafe"]
Order = Literal["K", "A", "C", "F"]


# ------ CONSTRUCTORS ------
class np_array(ExternalOpImplementation):
    _transform_id = "numpy.NP_ARRAY"
    _signature = SarusSignature(
        SarusParameter(
            name="object",
            annotation=ArrayLike,
            condition=DATASPEC | STATIC,
        ),
        SarusParameter(
            name="dtype",
            annotation=Optional[DTypeLike],
            default=None,
        ),
        SarusParameter(
            name="copy",
            annotation=bool,
            default=True,
        ),
        SarusParameter(
            name="order",
            annotation=Order,
            default="K",
        ),
        SarusParameter(
            name="subok",
            annotation=bool,
            default=False,
        ),
        SarusParameter(
            name="ndmin",
            annotation=int,
            default=0,
        ),
        SarusParameter(
            name="like",
            annotation=Optional[ArrayLike],
            default=None,
        ),
    )

    async def call(self, signature: SarusBoundSignature) -> Any:
        kwargs = await signature.collect_kwargs()
        if kwargs["like"] is None:
            del kwargs["like"]
        return np.array(**kwargs)


# ------ FUNCTIONS ------
class np_ceil(ExternalOpImplementation):
    _transform_id = "numpy.NP_CEIL"
    _signature = SarusSignature(
        SarusParameter(
            name="x",
            annotation=ArrayLike,
            condition=DATASPEC | STATIC,
        ),
        SarusParameter(
            name="out",
            annotation=Optional[ArrayLike],
            default=None,
            predicate=lambda x: x is None,
        ),
        SarusParameter(
            name="where",
            annotation=Optional[Union[bool, ArrayLike]],
            default=True,
        ),
        SarusParameter(
            name="casting",
            annotation=Casting,
            default="same_kind",
        ),
        SarusParameter(
            name="order",
            annotation=Order,
            default="K",
        ),
        SarusParameter(
            name="dtype",
            annotation=Optional[DTypeLike],
            default=None,
        ),
        SarusParameter(
            name="subok",
            annotation=bool,
            default=False,
        ),
    )

    async def call(self, signature: SarusBoundSignature) -> Any:
        x, kwargs = await signature.collect_kwargs_method()
        return np.ceil(x, **kwargs)


class np_floor(ExternalOpImplementation):
    _transform_id = "numpy.NP_FLOOR"
    _signature = SarusSignature(
        SarusParameter(
            name="x",
            annotation=ArrayLike,
            condition=DATASPEC | STATIC,
        ),
        SarusParameter(
            name="out",
            annotation=Optional[ArrayLike],
            default=None,
            predicate=lambda x: x is None,
        ),
        SarusParameter(
            name="where",
            annotation=Optional[Union[bool, ArrayLike]],
            default=True,
        ),
        SarusParameter(
            name="casting",
            annotation=Casting,
            default="same_kind",
        ),
        SarusParameter(
            name="order",
            annotation=Order,
            default="K",
        ),
        SarusParameter(
            name="dtype",
            annotation=Optional[DTypeLike],
            default=None,
        ),
        SarusParameter(
            name="subok",
            annotation=bool,
            default=False,
        ),
    )

    async def call(self, signature: SarusBoundSignature) -> Any:
        x, kwargs = await signature.collect_kwargs_method()
        return np.floor(x, **kwargs)


class np_mean(ExternalOpImplementation):
    _transform_id = "numpy.NP_MEAN"
    _signature = SarusSignature(
        SarusParameter(
            name="a",
            annotation=ArrayLike,
            condition=DATASPEC | STATIC,
        ),
        SarusParameter(
            name="axis",
            annotation=Optional[Union[int, Tuple[int]]],
            default=None,
        ),
        SarusParameter(
            name="dtype",
            annotation=Optional[DTypeLike],
            default=None,
        ),
        SarusParameter(
            name="out",
            annotation=Optional[ArrayLike],
            default=None,
            predicate=lambda x: x is None,
        ),
        SarusParameter(
            name="keepdims",
            annotation=bool,
            default=False,
        ),
        SarusParameter(
            name="where",
            annotation=Optional[Union[bool, ArrayLike]],
            default=True,
        ),
    )

    async def call(self, signature: SarusBoundSignature) -> Any:
        kwargs = await signature.collect_kwargs()
        return np.mean(**kwargs)


class np_std(ExternalOpImplementation):
    _transform_id = "numpy.NP_STD"
    _signature = SarusSignature(
        SarusParameter(
            name="a",
            annotation=ArrayLike,
            condition=DATASPEC | STATIC,
        ),
        SarusParameter(
            name="axis",
            annotation=Optional[Union[int, Tuple[int]]],
            default=None,
        ),
        SarusParameter(
            name="dtype",
            annotation=Optional[DTypeLike],
            default=None,
        ),
        SarusParameter(
            name="out",
            annotation=Optional[ArrayLike],
            default=None,
            predicate=lambda x: x is None,
        ),
        SarusParameter(
            name="ddof",
            annotation=int,
            default=0,
        ),
        SarusParameter(
            name="keepdims",
            annotation=bool,
            default=False,
        ),
        SarusParameter(
            name="where",
            annotation=Optional[Union[bool, ArrayLike]],
            default=True,
        ),
    )

    async def call(self, signature: SarusBoundSignature) -> Any:
        kwargs = await signature.collect_kwargs()
        return np.std(**kwargs)


class np_rand(ExternalOpImplementation):
    _transform_id = "numpy.NP_RAND"
    _signature = SarusSignature(
        SarusParameterArray(
            name="size",
            annotation=Optional[Union[int, List[int]]],
        ),
    )

    async def call(self, signature: SarusBoundSignature) -> Any:
        sizes = await signature.collect_args()
        return np.random.random_sample(sizes)


# ------ METHODS ------
class np_astype(ExternalOpImplementation):
    _transform_id = "numpy.NP_ASTYPE"
    _signature = SarusSignature(
        SarusParameter(
            name="this",
            annotation=np.ndarray,
            condition=DATASPEC | STATIC,
        ),
        SarusParameter(
            name="dtype",
            annotation=DTypeLike,
        ),
        SarusParameter(
            name="order",
            annotation=Order,
            default="K",
        ),
        SarusParameter(
            name="casting",
            annotation=Casting,
            default="unsafe",
        ),
        SarusParameter(
            name="subok",
            annotation=bool,
            default=False,
        ),
        SarusParameter(
            name="copy",
            annotation=bool,
            default=True,
        ),
    )

    async def call(self, signature: SarusBoundSignature) -> Any:
        this, kwargs = await signature.collect_kwargs_method()
        return this.astype(**kwargs)
